@@ -20,6 +20,8 @@ var EventEmitter = require('events').EventEmitter;
 var WebSocketFrame = require('./WebSocketFrame');
 var BufferList = require('../vendor/FastBufferList');
 var Validation = require('./Validation').Validation;
+var bufferAllocUnsafe = utils.bufferAllocUnsafe;
+var bufferFromString = utils.bufferFromString;
 
 // Connected, fully-open, ready to send and receive frames
 const STATE_OPEN = 'open';
@@ -73,8 +75,8 @@ function WebSocketConnection(socket, extensions, protocol, maskOutgoingPackets,
     // We re-use the same buffers for the mask and frame header for all frames
     // received on each connection to avoid a small memory allocation for each
     // frame.
-    this.maskBytes = new Buffer(4);
-    this.frameHeader = new Buffer(10);
+    this.maskBytes = bufferAllocUnsafe(4);
+    this.frameHeader = bufferAllocUnsafe(10);
 
     // the BufferList will handle the data streaming in
     this.bufferList = new BufferList();
@@ -176,7 +178,7 @@ function validateCloseReason(code) {
     if (code >= 1000 && code <= 2999) {
         // Codes from 1000 - 2999 are reserved for use by the protocol.  Only
         // a few codes are defined, all others are currently illegal.
-        return [1000, 1001, 1002, 1003, 1007, 1008, 1009, 1010, 1011].indexOf(code) !== -1;
+        return [1000, 1001, 1002, 1003, 1007, 1008, 1009, 1010, 1011, 1012, 1013, 1014].indexOf(code) !== -1;
     }
     if (code >= 3000 && code <= 3999) {
         // Reserved for use by libraries, frameworks, and applications.
@@ -209,7 +211,7 @@ WebSocketConnection.prototype._addSocketEventListeners = function() {
 // set or reset the keepalive timer when data is received.
 WebSocketConnection.prototype.setKeepaliveTimer = function() {
     this._debug('setKeepaliveTimer');
-    if (!this.config.keepalive) { return; }
+    if (!this.config.keepalive  || this.config.useNativeKeepalive) { return; }
     this.clearKeepaliveTimer();
     this.clearGracePeriodTimer();
     this._keepaliveTimeoutID = setTimeout(this._keepaliveTimerHandler, this.config.keepaliveInterval);
@@ -333,6 +335,11 @@ WebSocketConnection.prototype.processReceivedData = function() {
 
 WebSocketConnection.prototype.handleSocketError = function(error) {
     this._debug('handleSocketError: %j', error);
+    if (this.state === STATE_CLOSED) {
+		// See https://github.com/theturtle32/WebSocket-Node/issues/288
+        this._debug('  --- Socket \'error\' after \'close\'');
+        return;
+    }
     this.closeReasonCode = WebSocketConnection.CLOSE_REASON_ABNORMAL;
     this.closeDescription = 'Socket Error: ' + error.syscall + ' ' + error.code;
     this.connected = false;
@@ -341,7 +348,7 @@ WebSocketConnection.prototype.handleSocketError = function(error) {
     if (utils.eventEmitterListenerCount(this, 'error') > 0) {
         this.emit('error', error);
     }
-    this.socket.destroy(error);
+    this.socket.destroy();
     this._debug.printOutput();
 };
 
@@ -580,7 +587,7 @@ WebSocketConnection.prototype.processFrame = function(frame) {
                     // message now.  We also have to decode the utf-8 data
                     // for text frames after combining all the fragments.
                     var bytesCopied = 0;
-                    var binaryPayload = new Buffer(this.fragmentationSize);
+                    var binaryPayload = bufferAllocUnsafe(this.fragmentationSize);
                     var opcode = this.frameQueue[0].opcode;
                     this.frameQueue.forEach(function (currentFrame) {
                         currentFrame.binaryPayload.copy(binaryPayload, bytesCopied);
@@ -720,7 +727,7 @@ WebSocketConnection.prototype.send = function(data, cb) {
 };
 
 WebSocketConnection.prototype.sendUTF = function(data, cb) {
-    data = new Buffer(data.toString(), 'utf8');
+    data = bufferFromString(data.toString(), 'utf8');
     this._debug('sendUTF: %d bytes', data.length);
     var frame = new WebSocketFrame(this.maskBytes, this.frameHeader, this.config);
     frame.opcode = 0x01; // WebSocketOpcode.TEXT_FRAME
@@ -746,7 +753,7 @@ WebSocketConnection.prototype.ping = function(data) {
     frame.fin = true;
     if (data) {
         if (!Buffer.isBuffer(data)) {
-            data = new Buffer(data.toString(), 'utf8');
+            data = bufferFromString(data.toString(), 'utf8');
         }
         if (data.length > 125) {
             this._debug('WebSocket: Data for ping is longer than 125 bytes.  Truncating.');
@@ -839,7 +846,7 @@ WebSocketConnection.prototype.sendCloseFrame = function(reasonCode, description,
     frame.opcode = 0x08; // WebSocketOpcode.CONNECTION_CLOSE
     frame.closeStatus = reasonCode;
     if (typeof(description) === 'string') {
-        frame.binaryPayload = new Buffer(description, 'utf8');
+        frame.binaryPayload = bufferFromString(description, 'utf8');
     }
     
     this.sendFrame(frame, cb);
